﻿Type=Class
Version=5.2
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
#Region Preprocessor Directives
	#IgnoreWarnings: 9, 10, 12
#End Region

Private Sub Class_Globals
	Type    TextureObject(texture As lgTexture, x As Double, y As Double, w As Double, h As Double)
	Type    TexRegionObject(textureRegion As lgTextureRegion, x As Double, y As Double, w As Double, h As Double)
	Private lGdx                      As LibGDX
	Private Gl                        As lgGL
	Private Batch                     As lgSpriteBatch
	Private DefaultShader, BlurShader As lgShaderProgram
	Private FX                        As lgSpecialFX
	Private FXParam(4)                As lgSpecialFX_Param
	Private FirstBuffer, SecondBuffer As lgFrameBuffer
	Private BufferWidth, BufferHeight As Int
	Private BlurTR                    As lgTextureRegion
	Private DownSampling              As Double
	Private Intensity                 As Int
	Private textures(512)             As TextureObject
	Private texRegions(4096)          As TexRegionObject
	Private texPivot                  As Int
	Private texRegPivot               As Int
	Private lastTexture               As Int
	Private lastTexRegion             As Int
	Private invDownSampling           As Double
	Public  frameBuffer               As lgFrameBuffer
	Public  done                      as Boolean
End Sub

Sub Initialize(LibGDX As LibGDX, BatchObject As lgSpriteBatch, downSamplingValue As Double)
	lGdx             =  LibGDX
	DownSampling     =  downSamplingValue
	invDownSampling  =  1 / DownSampling
	Batch            =  BatchObject
	Intensity        =  100
	texPivot         =  0
	texRegPivot      =  0
	lastTexture      =  textures.Length - 1	
	lastTexRegion    =  texRegions.Length - 1	

	'Compiles the vertex and the fragment shaders	
	BlurShader.Pedantic = False
	BlurShader.InitializeWithFiles(lGdx.Files.Internal("shaderprog/vertex.txt"), _
	                               lGdx.Files.Internal("shaderprog/fragment_blur_kawase.txt"))

	'Is the shader program compiled?
	If Not(BlurShader.IsCompiled) Then
		Log("Could not compile shader: " & BlurShader.Log)
		Return
	End If

	'Logs any warning
	If BlurShader.Log.Length <> 0 Then
		Log(BlurShader.Log)
	End If	
End Sub

Sub Resize(Width As Int, Height As Int)
	BufferWidth = Width / DownSampling
	BufferHeight = Height / DownSampling
	If FirstBuffer.IsInitialized Then FirstBuffer.dispose
	FirstBuffer.Initialize(FirstBuffer.FORMAT_RGBA8888, BufferWidth, BufferHeight)
	If SecondBuffer.IsInitialized Then SecondBuffer.dispose
	SecondBuffer.Initialize(SecondBuffer.FORMAT_RGBA8888, BufferWidth, BufferHeight)
	If frameBuffer.IsInitialized Then frameBuffer.dispose
	frameBuffer.Initialize(frameBuffer.FORMAT_RGBA8888, BufferWidth, BufferHeight)	

	'Creates the region for the blurred texture
	BlurTR.InitializeWithTexture(SecondBuffer.ColorBufferTexture)
	BlurTR.Flip(True, False)

	'Creates the blur effect	
	Dim Fraction As Double = CalcFraction	
	Dim TexelWidth As Double = 1 / (BufferWidth * Fraction)
	Dim TexelHeight As Double = 1 / (BufferHeight * Fraction)
	For i = 0 To FXParam.Length - 1
		FXParam(i).Initialize
		FXParam(i).SetFloatParamArray("PixelSize", Array As Float(TexelWidth, TexelHeight))
	Next
	FXParam(0).SetFloatParam("Iteration", 0)
	FXParam(1).SetFloatParam("Iteration", 1)
	FXParam(2).SetFloatParam("Iteration", 1)	

	DefaultShader = Batch.CreateDefaultShader
	FX.ClearAllEffects
	FX.AddEffect(DefaultShader, Null, Null, FirstBuffer, "FX")
	FX.AddEffect(BlurShader, FXParam(0), FirstBuffer, SecondBuffer, "")
	FX.AddEffect(BlurShader, FXParam(1), SecondBuffer, FirstBuffer, "")
	FX.AddEffect(BlurShader, FXParam(2), FirstBuffer, SecondBuffer, "")	
End Sub

Sub Apply
	done = False
	Dim BatchWasDrawing = True As Boolean
	If Batch.IsDrawing Then	Batch.End Else BatchWasDrawing = False
	FX.Apply(Batch)
	Batch.Shader = DefaultShader
	If BatchWasDrawing Then Batch.Begin
	done = True
End Sub

Sub AddTexture(texture As lgTexture, x As Double, y As Double, width As Double, height As Double)
	textures(texPivot).texture = texture
	textures(texPivot).x = x
	textures(texPivot).y = y
	textures(texPivot).w = width
	textures(texPivot).h = height
	texPivot = texPivot + 1
	If texPivot >= textures.Length Then texPivot = lastTexture
End Sub

Sub AddTextureRegion(textureRegion As lgTextureRegion, x As Double, y As Double, width As Double, height As Double)
	texRegions(texRegPivot).textureRegion = textureRegion
	texRegions(texRegPivot).x = x
	texRegions(texRegPivot).y = y
	texRegions(texRegPivot).w = width
	texRegions(texRegPivot).h = height
	texRegPivot = texRegPivot + 1
	If texRegPivot >= texRegions.Length Then texRegPivot = lastTexRegion
End Sub

Private Sub FX_Draw	
	Gl.glClear(Gl.GL10_COLOR_BUFFER_BIT) 'Gl.glClearColor(0, 0, 0, 0)
	Dim i = 0 As Int
	Do While i < texPivot
		Batch.DrawTex2(textures(i).texture, textures(i).x * invDownSampling, textures(i).y * invDownSampling, textures(i).w * invDownSampling, textures(i).h * invDownSampling)
		i = i + 1
	Loop
	texPivot = 0
	
	i = 0
	Do While i < texRegPivot
		Batch.DrawRegion2(texRegions(i).textureRegion, texRegions(i).x * invDownSampling, texRegions(i).y * invDownSampling, texRegions(i).w * invDownSampling, texRegions(i).h * invDownSampling)
		i = i + 1
	Loop
	texRegPivot = 0
End Sub

Sub Draw(x As Double, y As Double, width As Double, height As Double)
	Dim BatchWasDrawing As Boolean
	If Batch.IsDrawing = False Then Batch.Begin Else BatchWasDrawing = True		
	Batch.DrawRegion2(BlurTR, 0, 0, width, height)
	If Batch.IsDrawing And BatchWasDrawing = False Then Batch.End	
End Sub

Sub DrawFB(x As Double, y As Double, width As Double, height As Double)
	Dim BatchWasDrawing As Boolean
	If Batch.IsDrawing = False Then Batch.Begin Else BatchWasDrawing = True	
	frameBuffer.Begin
	Batch.DrawRegion2(BlurTR, 0, 0, width, height)
	frameBuffer.End	
	If Batch.IsDrawing And BatchWasDrawing = False Then Batch.End
End Sub

Sub Dispose
	DefaultShader.dispose
	BlurShader.dispose
	FirstBuffer.dispose
	SecondBuffer.dispose
	frameBuffer.dispose
	FX.dispose	
End Sub

Sub SetIntensity(Value As Int)
	'Changes the blur strength by modifying the size of texels
	Intensity = Value
	Dim Fraction As Double = CalcFraction	
	Dim TexelWidth As Double = 1 / (BufferWidth * Fraction)
	Dim TexelHeight As Double = 1 / (BufferHeight * Fraction)
	For i = 0 To FXParam.Length - 1		
		FXParam(i).SetFloatParamArray("PixelSize", Array As Float(TexelWidth, TexelHeight))
	Next
End Sub

Private Sub CalcFraction As Double
	Dim Strength As Int = 100 - Intensity
	Strength = Strength * (Strength / 100)
	Return Strength / (20 / (DownSampling - 1)) + 1
End Sub